# -*- coding: utf-8 -*-

"""Main module."""
import shlex
import subprocess as sp
import typing as T
from pathlib import Path
from tempfile import NamedTemporaryFile

from ninjadog.constants import PUG_CLI_PATH
from ninjadog.utils import jsonify


def render(string: str = '',
           filepath: T.Union[Path, str] = None,
           context: T.Any = None,
           pretty: bool = False,
           pug_cli_path: T.Union[Path, str] = None) -> str:
    """
    Render a pug template through the pug cli.
    
    Args:
        string: a string in pug syntax to be rendered
        filepath: the path to a pug template
        context: the data to be passed to the template
        pretty: pretty html output
        pug_cli_path: path to the pug cli

    Returns: rendered html

    """

    if pug_cli_path is None and PUG_CLI_PATH is None:
        msg = "the pug command was not found. Did you install it?\n" \
              "brew install npm\n" \
              "npm install -g pug-cli"
        raise ValueError(msg)
    elif pug_cli_path is None:
        pug_cli_path = PUG_CLI_PATH

    # create Path object if filepath argument is given
    # Path() is idempotent so this shouldn't make any difference
    # if the filepath argument is of type Path
    filepath = Path(filepath) if filepath else filepath

    # if filepath is given instead of a string argument,
    # return render of string
    if filepath and not string:
        with open(filepath) as fp:
            return render(fp.read(),
                          filepath,
                          context=context,
                          pretty=pretty,
                          pug_cli_path=pug_cli_path)

    with NamedTemporaryFile('w') as fp:
        fp.write(string)
        fp.seek(0)

        cmd = shlex.quote(str((Path(pug_cli_path).absolute())))
        path = f'-p {shlex.quote(str(filepath))}' if filepath else ''
        pretty_print = '-P' if pretty else ''

        if context is None:
            context_arg = ''
        elif isinstance(context, str):
            context_arg = f'-O {shlex.quote(context)}'
        else:
            context_arg = f'-O {shlex.quote(jsonify(context))}'

        input_file = shlex.quote(fp.name)

        return sp.run(f'{cmd} {context_arg} {path} {pretty_print} < {input_file}',
                      shell=True,
                      stdout=sp.PIPE,
                      cwd=filepath.parent if filepath else None,
                      ).stdout.decode('utf8')



# -*- coding: utf-8 -*-

"""Top-level package for ninjadog."""

__author__ = """Stephan Fitzpatrick"""
__email__ = 'knowsuchagency@gmail.com'
__version__ = '0.2.0'

from ninjadog.ninjadog import render
from ninjadog.ext.jinja2 import jinja2_renderer



import typing as T
from functools import partial
from json import dumps
from pathlib import Path

from .constants import EXTENSION_PATT

jsonify = partial(dumps, skipkeys=True, default=lambda _: '', ensure_ascii=False)



import re
import subprocess as sp
import typing as T
from pathlib import Path

EXTENSION_PATT: re.sre_parse.Pattern
PUG_CLI_PATH: T.Optional[Path]

EXTENSION_PATT = re.compile('(?:extends|include)\s+(.+)')
PUG_CLI_PATH = Path(sp.run(('which', 'pug'), stdout=sp.PIPE).stdout.decode('utf8').strip()) \
    if sp.run(('which', 'npm'), stdout=sp.DEVNULL).returncode == 0 else None




from pathlib import Path

from pyramid_jinja2 import *

from ninjadog import jinja2_renderer, render


# TODO: See if it might not make more to just render all pug templates in the static directory first, then render them in jinja2 as normal as a configuration option


class PugTemplateRenderer(Jinja2TemplateRenderer):
    """
    Renders templates that have both pug and jinja2
    syntax.
    
    Conforms to `IRenderer <http://docs.pylonsproject.org/projects/pyramid/en/latest/api/interfaces.html#pyramid.interfaces.IRenderer>`_
    interface.
    """
    def __call__(self, value, system):
        try:
            system.update(value)
        except (TypeError, ValueError) as ex:
            raise ValueError('renderer was passed non-dictionary '
                             'as value: %s' % str(ex))
        template = self.template_loader()
        jinja2_string = template.render(system)
        with_pug = render(jinja2_string, filepath=template.filename, context=system)
        return with_pug


class PugRendererFactory(object):
    """
    Renderer factory conforms to
    `IRendererFactory <http://docs.pylonsproject.org/projects/pyramid/en/latest/api/interfaces.html#pyramid.interfaces.IRendererFactory>`_
    interface.
    """
    environment = None

    def __call__(self, info):
        name, package = info.name, info.package

        def template_loader():
            # attempt to turn the name into a caller-relative asset spec
            if ':' not in name and package is not None:
                try:
                    name_with_package = '%s:%s' % (package.__name__, name)
                    return self.environment.get_template(name_with_package)
                except TemplateNotFound:
                    pass

            return self.environment.get_template(name)

        return PugTemplateRenderer(template_loader)


def add_pug_renderer(config, name, settings_prefix='jinja2.', package=None):
    """
    This function is added as a method of a :term:`Configurator`, and
    should not be called directly.  Instead it should be called like so after
    ``pyramid_jinja2`` has been passed to ``config.include``:
    .. code-block:: python
       config.add_jinja2_renderer('.html', settings_prefix='jinja2.')
    It will register a new renderer, loaded from settings at the specified
    ``settings_prefix`` prefix. This renderer will be active for files using
    the specified extension ``name``.
    """
    renderer_factory = PugRendererFactory()
    config.add_renderer(name, renderer_factory)

    package = package or config.package
    resolver = DottedNameResolver(package=package)

    def register():
        registry = config.registry
        settings = config.get_settings()

        loader_opts = parse_loader_options_from_settings(
            settings,
            settings_prefix,
            resolver.maybe_resolve,
            package,
        )
        env_opts = parse_env_options_from_settings(
            settings,
            settings_prefix,
            resolver.maybe_resolve,
            package,
        )
        env = create_environment_from_options(env_opts, loader_opts)
        renderer_factory.environment = env

        registry.registerUtility(env, IJinja2Environment, name=name)

    config.action(
        ('pug-renderer', name), register, order=ENV_CONFIG_PHASE)


def includeme(config):
    config.add_directive('add_pug_renderer', add_pug_renderer)
    config.add_pug_renderer('.pug')



import typing as T
from functools import partial
from pathlib import Path

from jinja2 import Environment
from jinja2.ext import Extension

from ninjadog import render as _render


class PugPreprocessor(Extension):
    """
    Jinja2 preprocessor that renders pug to html.
    """

    def preprocess(self, source, name, filename=None):
        """
        Render pug string to html.
        
        Args:
            source: string to be rendered
            name: the name of the template
            filename: the filepath of the template

        Returns: rendered html

        """
        return _render(source, filepath=Path(filename) if filename else None)


def jinja2_renderer(string: str = '',
                    filepath: Path = None,
                    context: dict = None,
                    pretty: bool = False,
                    pug_cli_path: T.Union[Path, str] = None,
                    ) -> str:
    """
    Renders pug and jinja2 syntax formatted string to html.
    
    Jinja2 will be rendered first and can accept any Python
    object as its context. 
    
    The pug-template engine will only receive json-serializable
    data, however.
    
    Args:
        string: the string to be rendered
        filepath: the path to the template to be rendered
        context: the data to be passed to the template engines
        pretty: pretty html output
        pug_cli_path: Path to the pug cli

    Returns: html string

    """
    # lock arguments
    pug_renderer = partial(_render,
                           context=context,
                           filepath=filepath,
                           pretty=pretty,
                           pug_cli_path=pug_cli_path)

    # initialize jinja2 environment
    env = Environment()
    env.globals = context if context else {}

    return env.from_string(pug_renderer(string)).render()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `ninjadog` package."""

# TODO: test raises ValueError when pug cli can't be found and not passed explicitly to renderer

import pytest


def test_npm_installed():
    from subprocess import run
    assert run(('which', 'npm')).returncode == 0, 'npm must be installed'


def test_pug_cli_exists():
    from ninjadog.constants import PUG_CLI_PATH
    assert PUG_CLI_PATH.exists()


def test_hello_world():
    import ninjadog
    assert ninjadog.render('h1 hello world') == '<h1>hello world</h1>'


def test_jinja2_template_syntax():
    import ninjadog
    assert ninjadog.render('h1 hello {{ name }}!') == '<h1>hello {{ name }}!</h1>'


def test_context():
    import ninjadog
    context = {'name': 'Derp'}
    assert ninjadog.render('h1 hello #{ name }', context=context) == '<h1>hello Derp</h1>'
    assert ninjadog.render("h1= name", context=context) == '<h1>Derp</h1>'


def test_conditional():
    from textwrap import dedent
    import ninjadog
    string = dedent("""
    if name == 'sam'
        h1 hello #{ name }
    """)
    assert ninjadog.render(string, context={'name': 'sam'}) == '<h1>hello sam</h1>'

    string = dedent("""
    if person.name == 'sam'
        h1 hello #{ person.name }
    """)
    assert ninjadog.render(string, context={'person': {'name': 'sam'}}) == '<h1>hello sam</h1>'


def test_jinja2_renderer():
    from ninjadog import jinja2_renderer
    assert jinja2_renderer('h1 hello {{ name }}', context={'name': 'fred'}) == '<h1>hello fred</h1>'


def test_render_no_string_argument():
    from tempfile import NamedTemporaryFile
    import ninjadog
    string = 'h1 hello'
    with NamedTemporaryFile('w+') as tempfile:
        tempfile.write(string)
        tempfile.seek(0)
        assert ninjadog.render(filepath=tempfile.name) == ninjadog.render(string) == '<h1>hello</h1>'


def test_jinja2_render_no_string_argument():
    from tempfile import NamedTemporaryFile
    import ninjadog
    string = 'h1 hello {{ name }}'
    context = {'name': 'ash'}
    with NamedTemporaryFile('w+') as tempfile:
        tempfile.write(string)
        tempfile.seek(0)
        assert ninjadog.jinja2_renderer(filepath=tempfile.name, context=context) \
               == ninjadog.jinja2_renderer(string, context=context) \
               == '<h1>hello ash</h1>'


def test_jinja2_syntax_with_pug_syntax():
    from textwrap import dedent
    from ninjadog import jinja2_renderer
    string = dedent("""
    if person.name == "Bob"
        h1 Hello Bob
    else
        h1 My name is #{ person.name }

    p The persons's uppercase name is {{ person.get('name').upper() }}
    p The person's name is #{ person.name }

    if animal
        h1 This should not output
    else
        p animal value is false
    """).strip()

    context = {'person': {'name': 'Bob'}, 'animal': None}

    expected_output = dedent("""
    <h1>Hello Bob</h1>
    <p>The persons's uppercase name is BOB</p>
    <p>The person's name is Bob</p>
    <p>animal value is false</p>
    """).strip()

    actual_output = jinja2_renderer(string, context=context, pretty=True).strip()

    assert expected_output == actual_output



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('requirements.txt') as requirements_file:
    requirements = requirements_file.readlines()

setup_requirements = [
    'pytest-runner',
]

test_requirements = [
    'pytest',
]

setup(
    name='ninjadog',
    version='0.2.0',
    description="Pug template support in Python",
    long_description=readme,
    author="Stephan Fitzpatrick",
    author_email='knowsuchagency@gmail.com',
    url='https://github.com/knowsuchagency/ninjadog',
    packages=find_packages(include=['ninjadog']),
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='ninjadog pug jinja2 html templating',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
)
